@@ -179,6 +179,9 @@ static bool elim_tripleton(const char *msg,
 	      return (true);
 	    
 	    // have to adjust various induction variables
+ 	    kcolx = mcstrt[icolx] + (kcolx - kcsx);
+ 	    kcsx = mcstrt[icolx];			
+ 	    kcex = mcstrt[icolx] + hincol[icolx];
 	    kcoly = mcstrt[icoly] + (kcoly - kcs);
 	    kcs = mcstrt[icoly];			// do this for ease of debugging
 	    kce = mcstrt[icoly] + hincol[icoly];
@@ -325,7 +328,7 @@ const CoinPresolveAction *tripleton_action::presolve(CoinPresolveMatrix *prob,
 # endif
   int nactions = 0;
 
-  int *zeros	= new int[ncols];
+  int *zeros	= prob->usefulColumnInt_; //new int[ncols];
   memset(zeros,0,ncols*sizeof(int));
   int nzeros	= 0;
 
@@ -625,7 +628,7 @@ const CoinPresolveAction *tripleton_action::presolve(CoinPresolveMatrix *prob,
     }
   }
 
-  delete[]zeros;
+  //delete[]zeros;
   deleteAction(actions,action*);
 
   if (prob->tuning_) {
@@ -1044,7 +1047,7 @@ void check_tripletons(const CoinPresolveAction * paction)
     check_tripletons(paction->next);
     
     if (strcmp(paction0->name(), "tripleton_action") == 0) {
-      const tripleton_action *daction = (const tripleton_action *)paction0;
+      const tripleton_action *daction = reinterpret_cast<const tripleton_action *>(paction0);
       for (int i=daction->nactions_-1; i>=0; --i) {
 	int icolx = daction->actions_[i].icolx;
 	int icoly = daction->actions_[i].icoly;
